@@ -58,6 +58,7 @@ class BaseDeliverScript(BaseLoopScript, DeliverScriptProtocol):
     MAX_PROCESSED_ITEMS = float("inf")
 
     SEEN_ITEMS_CLASS: Type[DupesFilterProtocol] = SqliteDictDupesFilter
+    strict_max_time = False
 
     def __init__(self):
         super().__init__()
@@ -107,6 +108,8 @@ class BaseDeliverScript(BaseLoopScript, DeliverScriptProtocol):
     def process_spider_jobs(self, scrapername: str, only_finished: bool = True) -> bool:
         if self.total_items_count >= self.MAX_PROCESSED_ITEMS:
             return False
+        if self.strict_max_time and self.is_max_time_ran_out():
+            return False
         target_tags = self.get_target_tags()
         for sj in self.get_delivery_spider_jobs(scrapername, target_tags, only_finished):
             if sj.key in self._all_jobs_to_tag:
@@ -116,6 +119,8 @@ class BaseDeliverScript(BaseLoopScript, DeliverScriptProtocol):
                 self._all_jobs_to_tag.append(sj.key)
             if self.total_items_count >= self.MAX_PROCESSED_ITEMS:
                 return False
+            if self.strict_max_time and self.is_max_time_ran_out():
+                return False
         return True
 
     def get_item_unique_key(self, item: Item) -> str:
@@ -162,11 +167,15 @@ class BaseDeliverScript(BaseLoopScript, DeliverScriptProtocol):
                 return True
         return False
 
+    def close_files(self):
+        pass
+
     def on_close(self):
         if self.loop_mode:
             for scrapername in self.args.scrapername:
                 _LOG.info(f"Processing remaining spider {scrapername}")
                 self.process_spider_jobs(scrapername, only_finished=False)
+        self.close_files()
         jobs_count = len(self._all_jobs_to_tag)
         _LOG.info(f"Processed a total of {jobs_count} jobs.")
         _LOG.info(f"Processed a total of {self.total_items_count} items.")
