@@ -26,7 +26,7 @@ except ImportError:
     OLD_YAML = False
 
 
-from shub_workflow.script import JobKey, JobDict
+from shub_workflow.script import JobKey, JobDict, Outcome
 from shub_workflow.base import WorkFlowManager
 from shub_workflow.graph.task import (
     JobGraphDict,
@@ -62,10 +62,9 @@ class GraphManager(WorkFlowManager):
         # Ensure jobs are traversed in the same order as they went pending.
         self.__pending_jobs: Dict[TaskId, PendingJobDict] = OrderedDict()
         self.__running_jobs: Dict[TaskId, JobKey] = OrderedDict()
+        self.__completed_jobs: Dict[TaskId, Tuple[JobKey, Outcome]] = dict()
         self._available_resources: Dict[Resource, ResourceAmmount] = {}  # map resource : ammount
-        self._acquired_resources: DefaultDict[Resource, List[Tuple[TaskId, ResourceAmmount]]] = defaultdict(
-            list
-        )  # map resource : list of (job, ammount)
+        self._acquired_resources: DefaultDict[Resource, List[Tuple[TaskId, ResourceAmmount]]] = defaultdict(list)
         self.__tasks: Dict[TaskId, BaseTask] = {}
         super(GraphManager, self).__init__()
         self.__start_time: DefaultDict[TaskId, float] = defaultdict(time)
@@ -97,29 +96,25 @@ class GraphManager(WorkFlowManager):
     def on_start(self):
         if not self.jobs_graph:
             self.argparser.error("Jobs graph configuration is empty.")
-        self.__starting_jobs = self.args.starting_job or self.__starting_jobs
         if not self.__starting_jobs:
             self.argparser.error("You must provide either --starting-job or --root-jobs.")
         self._fill_available_resources()
-        self._setup_starting_jobs([])
+        self._setup_starting_jobs()
         self.workflow_loop_enabled = True
         logger.info("Starting '%s' workflow", self.name)
 
-    def _setup_starting_jobs(self, ran_tasks: List[TaskId], candidates: Optional[List[TaskId]] = None):
-        candidates = candidates or self.__starting_jobs
-        for taskid in candidates:
-            if taskid in ran_tasks:
-                logger.info(
-                    "Task %s already done %s.",
-                    taskid,
-                    tuple(self.__tasks[taskid].get_scheduled_jobs()),
-                )
-                next_tasks = [t.task_id for t in self.__tasks[taskid].get_next_tasks()]
-                if next_tasks:
-                    self._setup_starting_jobs(ran_tasks, next_tasks)
-            else:
-                self._add_initial_pending_job(taskid)
-                logger.info("Starting at task %s", taskid)
+    def _setup_starting_jobs(self) -> None:
+        for taskid in self.__starting_jobs:
+            wait_for: List[TaskId] = self.get_jobdict(taskid).get("wait_for", [])
+            self._add_pending_job(taskid, wait_for=tuple(wait_for))
+
+        for taskid in list(self.__pending_jobs.keys()):
+            if taskid in self.__completed_jobs:
+                jobid, outcome = self.__completed_jobs[taskid]
+                self.__pending_jobs.pop(taskid)
+                self._check_completed_job(taskid, jobid, outcome)
+            elif taskid in self.__running_jobs:
+                self.__pending_jobs.pop(taskid)
 
     def _fill_available_resources(self):
         """
@@ -149,10 +144,6 @@ class GraphManager(WorkFlowManager):
             return self.jobs_graph.pop(job)
         return self.jobs_graph[job]
 
-    def _add_initial_pending_job(self, job: TaskId):
-        wait_for: List[TaskId] = self.get_jobdict(job).get("wait_for", [])
-        self._add_pending_job(job, wait_for=tuple(wait_for))
-
     def _add_pending_job(self, job: TaskId, wait_for=(), is_retry=False):
         self._maybe_add_on_finish_default(job, is_retry)
         if job in self.args.skip_job:
@@ -174,28 +165,28 @@ class GraphManager(WorkFlowManager):
             # Split parallelized task into N parallel jobs.
             basejobconf = self.get_jobdict(job, pop=True)
             for i in range(parallelization):
-                job_unit = TaskId("%s_%i" % (job, i))
-                job_unit_conf = deepcopy(basejobconf)
-                job_unit_conf["origin"] = job
-                job_unit_conf["index"] = i
+                subtask_id = TaskId("%s.%i" % (job, i))
+                subtask_conf = deepcopy(basejobconf)
+                subtask_conf["origin"] = job
+                subtask_conf["index"] = i
 
-                for _, nextjobs in job_unit_conf["on_finish"].items():
+                for _, nextjobs in subtask_conf["on_finish"].items():
                     if i != 0:  # only job 0 will conserve finish targets
                         for nextjob in copy(nextjobs):
                             if nextjob != "retry":
                                 if nextjob in self.jobs_graph:
-                                    self.get_jobdict(nextjob)["wait_for"].append(job_unit)
+                                    self.get_jobdict(nextjob)["wait_for"].append(subtask_id)
                                     if nextjob in self.__pending_jobs:
-                                        self.__pending_jobs[nextjob]["wait_for"].add(job_unit)
+                                        self.__pending_jobs[nextjob]["wait_for"].add(subtask_id)
                                 else:
                                     for i in range(parallelization):
-                                        nextjobp = TaskId("%s_%i" % (job, i))
-                                        self.get_jobdict(nextjobp)["wait_for"].append(job_unit)
+                                        nextjobp = TaskId("%s.%i" % (job, i))
+                                        self.get_jobdict(nextjobp)["wait_for"].append(subtask_id)
                                         if nextjobp in self.__pending_jobs:
-                                            self.__pending_jobs[nextjobp]["wait_for"].add(job_unit)
+                                            self.__pending_jobs[nextjobp]["wait_for"].add(subtask_id)
                                 nextjobs.remove(nextjob)
-                self.jobs_graph[job_unit] = job_unit_conf
-                self.__pending_jobs[job_unit] = {
+                self.jobs_graph[subtask_id] = subtask_conf
+                self.__pending_jobs[subtask_id] = {
                     "wait_for": set(wait_for),
                     "is_retry": is_retry,
                     "origin": job,
@@ -207,10 +198,10 @@ class GraphManager(WorkFlowManager):
                     if other in self.__pending_jobs:
                         self.__pending_jobs[other]["wait_for"].discard(job)
                     for i in range(parallelization):
-                        job_unit = TaskId("%s_%i" % (job, i))
-                        oconf["wait_for"].append(job_unit)
+                        subtask_id = TaskId("%s.%i" % (job, i))
+                        oconf["wait_for"].append(subtask_id)
                         if other in self.__pending_jobs:
-                            self.__pending_jobs[other]["wait_for"].add(job_unit)
+                            self.__pending_jobs[other]["wait_for"].add(subtask_id)
 
     def add_argparser_options(self):
         super(GraphManager, self).add_argparser_options()
@@ -379,29 +370,40 @@ class GraphManager(WorkFlowManager):
         logger.warning("No more retries for failed job %s (outcome: %s)", job, outcome)
         return False
 
-    def check_running_jobs(self):
+    def check_running_jobs(self) -> None:
         for task_id, jobid in list(self.__running_jobs.items()):
             outcome = self.is_finished(jobid)
             if outcome is not None:
-                logger.info('Job "%s/%s" (%s) finished', self.name, task_id, jobid)
-                for st in self.__pending_jobs.values():
-                    st["wait_for"].discard(task_id)
-                for conf in self.jobs_graph.values():
-                    if task_id in conf.get("wait_for", []):
-                        conf["wait_for"].remove(task_id)
-                for nextjob in self._get_next_jobs(task_id, outcome):
-                    if nextjob == "retry":
-                        self.handle_retry(task_id, outcome)
-                    elif nextjob in self.__pending_jobs:
-                        logger.error("Job %s already pending", nextjob)
-                    else:
-                        wait_for = self.get_jobdict(nextjob).get("wait_for", [])
-                        self._add_pending_job(nextjob, wait_for)
-                self._release_resources(task_id)
+                self._check_completed_job(task_id, jobid, outcome)
                 self.__running_jobs.pop(task_id)
             else:
                 logger.info("Job %s (%s) still running", task_id, jobid)
 
+    def _check_completed_job(self, task_id: TaskId, jobid: JobKey, outcome: Outcome):
+        will_retry = False
+        logger.info('Job "%s/%s" (%s) finished', self.name, task_id, jobid)
+        for nextjob in self._get_next_jobs(task_id, outcome):
+            if nextjob == "retry":
+                will_retry = self.handle_retry(task_id, outcome)
+                if not will_retry:
+                    self.bad_outcome_hook(task_id, jobid, outcome)
+            elif nextjob in self.__pending_jobs:
+                logger.error("Job %s already pending", nextjob)
+            else:
+                wait_for = self.get_jobdict(nextjob).get("wait_for", [])
+                self._add_pending_job(nextjob, wait_for)
+        self._release_resources(task_id)
+        if not will_retry:
+            self.__completed_jobs[task_id] = jobid, outcome
+            for st in self.__pending_jobs.values():
+                st["wait_for"].discard(task_id)
+            for conf in self.jobs_graph.values():
+                if task_id in conf.get("wait_for", []):
+                    conf["wait_for"].remove(task_id)
+
+    def bad_outcome_hook(self, task_id: TaskId, jobid: JobKey, outcome: Outcome):
+        pass
+
     def _try_acquire_resources(self, job: TaskId) -> bool:
         result = True
         task_id = self.get_jobdict(job).get("origin", job)
@@ -437,7 +439,7 @@ class GraphManager(WorkFlowManager):
 
         return on_finish
 
-    def _get_next_jobs(self, job: TaskId, outcome: OnFinishKey) -> OnFinishTarget:
+    def _get_next_jobs(self, job: TaskId, outcome: Outcome) -> OnFinishTarget:
         if self.args.only_starting_jobs:
             return []
         on_finish = self.get_jobdict(job)["on_finish"]
@@ -454,8 +456,22 @@ class GraphManager(WorkFlowManager):
     def pending_jobs(self) -> Dict[TaskId, PendingJobDict]:
         return self.__pending_jobs
 
+    def get_job_taskid(self, job: JobDict) -> Optional[TaskId]:
+        task_id = self.get_keyvalue_job_tag("TASK_ID", job["tags"])
+        if task_id is not None:
+            return TaskId(task_id)
+        return None
+
     def resume_running_job_hook(self, job: JobDict):
-        pass
+        task_id = self.get_job_taskid(job)
+        if task_id is not None:
+            self.__running_jobs[task_id] = job["key"]
+            origin_task_id = TaskId(task_id.rsplit(".", 1)[0])
+            self.__tasks[origin_task_id].append_jobid(job["key"])
 
     def resume_finished_job_hook(self, job: JobDict):
-        pass
+        task_id = self.get_job_taskid(job)
+        if task_id is not None:
+            self.__completed_jobs[task_id] = job["key"], Outcome(job["close_reason"])
+            origin_task_id = TaskId(task_id.rsplit(".", 1)[0])
+            self.__tasks[origin_task_id].append_jobid(job["key"])
