@@ -14,6 +14,7 @@ from typing import List, NewType, Optional, Tuple, Generator, Dict, Union, Any,
 from typing_extensions import TypedDict, NotRequired, Protocol
 
 from scrapy.utils.misc import load_object
+from scrapy.spiderloader import SpiderLoader
 from scrapy.statscollectors import StatsCollector
 from scrapinghub import ScrapinghubClient, DuplicateJobError
 from scrapinghub.client.jobs import Job, JobMeta
@@ -31,6 +32,7 @@ logger.setLevel(logging.INFO)
 
 
 JobKey = NewType("JobKey", str)  # ScrapyCloud job key
+Outcome = NewType("Outcome", str)
 
 
 class JobDict(TypedDict):
@@ -138,7 +140,7 @@ class BaseScriptProtocol(ArgumentParserScriptProtocol, Protocol):
         ...
 
     @abc.abstractmethod
-    def is_finished(self, jobkey: JobKey) -> Optional[str]:
+    def is_finished(self, jobkey: JobKey) -> Optional[Outcome]:
         ...
 
     @abc.abstractmethod
@@ -162,9 +164,10 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
         self.client = ScrapinghubClient(max_retries=100)
         self.close_reason: Optional[str] = None
         self.__flow_tags: List[str] = []
+        self.project_settings = get_project_settings()
+        self.loader = SpiderLoader(self.project_settings)
         super().__init__()
         self.set_flow_id_name(self.args)
-        self.project_settings = get_project_settings()
 
     def append_flow_tag(self, tag: str):
         """
@@ -401,7 +404,7 @@ class BaseScript(ArgumentParserScript, BaseScriptProtocol):
             return True
         return False
 
-    def is_finished(self, jobkey: JobKey) -> Optional[str]:
+    def is_finished(self, jobkey: JobKey) -> Optional[Outcome]:
         """
         Checks whether a job is finished. if so, return close_reason. Otherwise return None.
         """
@@ -509,9 +512,12 @@ class BaseLoopScript(BaseScript, BaseLoopScriptProtocol):
     def base_loop_tasks(self):
         pass
 
+    def is_max_time_ran_out(self) -> bool:
+        return self.args.max_running_time > 0 and time.time() - self.__start_time > self.args.max_running_time
+
     def __base_loop_tasks(self):
         self.base_loop_tasks()
-        if self.args.max_running_time and time.time() - self.__start_time > self.args.max_running_time:
+        if self.is_max_time_ran_out():
             logger.info("Time limit reached. Closing.")
             self.workflow_loop_enabled = False
 
