@@ -5,7 +5,7 @@ import hashlib
 from typing import Optional
 
 from tenacity import retry, retry_if_exception_type, before_sleep_log, stop_after_attempt, wait_fixed
-from scrapinghub.client.exceptions import ServerError
+from scrapinghub.client.exceptions import ScrapinghubAPIError, ServerError
 from requests.exceptions import ReadTimeout, ConnectionError, HTTPError
 
 
@@ -62,7 +62,7 @@ ONE_MIN_IN_S = 60
 
 
 dash_retry_decorator = retry(
-    retry=retry_if_exception_type((ServerError, ReadTimeout, ConnectionError, HTTPError)),
+    retry=retry_if_exception_type((ScrapinghubAPIError, ServerError, ReadTimeout, ConnectionError, HTTPError)),
     before_sleep=before_sleep_log(logger, logging.ERROR),
     reraise=True,
     stop=stop_after_attempt(MINS_IN_A_DAY),
